/*! \file TransitionMatrix.h
 *
 * \brief Representation of the Transition Matrix.
 */

#ifndef INCLUDE_TRANSITIONMATRIX_H_
#define INCLUDE_TRANSITIONMATRIX_H_

/**
 * \brief Exception for unrecognized file formats.
 */
class UnrecognizedFormatException: public std::exception {
protected:
  //! Description of the problem.
  std::string message;
public:
  /**
   * \brief Exception instance constructor.
   *
   * \param problem: `string` Description of the problem that occurred.
   */
  UnrecognizedFormatException(std::string problem) { message = problem; }
  /**
   * \brief Exception message.
   */
  virtual const char* what() const throw() {
    return message.c_str();
  }
};

/*! \brief Class to represent the Transition Matrix.
 */
class TransitionMatrix {
 protected:
  //! Matrix type identifier.
  int is;
  //! Maximum field expansion order.
  int l_max;
  //! Wave number in scale units.
  double vk;
  //! External medium refractive index.
  double exri;
  //! Vectorized matrix elements.
  std::complex<double> *elements;
  //! Sphere radius.
  double sphere_radius;
  //! Matrix shape
  int *shape;

  /*! \brief Write the Transition Matrix to HDF5 binary output.
   *
   * \param file_name: `string` Name of the binary configuration data file.
   */
  void write_hdf5(std::string file_name);
  
  /*! \brief Write the Transition Matrix to legacy binary output.
   *
   * \param file_name: `string` Name of the binary configuration data file.
   */
  void write_legacy(std::string file_name);
 public:
  /*! \brief Transition Matrix instance constructor for single sphere.
   *
   * \param _lm: `int` Maximum field expansion order.
   * \param _vk: `double` Wave number in scale units.
   * \param _exri: `double` External medium refractive index.
   * \param _rmi: Matrix of complex.
   * \param _rei: Matrix of complex.
   * \param _sphere_radius: `double` Radius of the sphere.
   */
  TransitionMatrix(
		   int _lm, double _vk, double _exri, std::complex<double> **_rmi,
		   std::complex<double> **_rei, double _sphere_radius
		   );

  /*! \brief Transition Matrix instance constructor for a cluster of spheres.
   *
   * \param _nlemt: `int` Size of the matrix (2 * LE * (LE + 2)).
   * \param _lm: `int` Maximum field expansion order.
   * \param _vk: `double`
   * \param _exri: `double`
   * \param _am0m: Matrix of complex.
   */
  TransitionMatrix(int _nlemt, int _lm, double _vk, double _exri, std::complex<double> **am0m);

  /*! \brief Transition Matrix instance destroyer.
   */
  ~TransitionMatrix();
  
  /*! \brief Write the Transition Matrix to a binary file.
   *
   * \param file_name: `string` Name of the file to be written.
   * \param mode: `string` Binary encoding. Can be one of ["LEGACY", "HDF5"] . Optional
   * (default is "LEGACY").
   */
  void write_binary(std::string file_name, std::string mode="LEGACY");
};

#endif
